@@ -110,7 +110,7 @@ sub ProcessAPP12($$$);
         Name => 'Copyright',
         Format => 'undef[$size-16]',
         Groups => { 2 => 'Author' },
-        ValueConv => '$self->Unicode2Byte($val)',
+        ValueConv => '$self->Unicode2Charset($val)',
     },
 );
 
@@ -185,7 +185,7 @@ APP12 meta information.
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
