@@ -18,7 +18,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.17';
+$VERSION = '1.19';
 
 %Image::ExifTool::Casio::Main = (
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
@@ -97,7 +97,7 @@ $VERSION = '1.17';
         ValueConv => '$val / 1000', #4
         ValueConvInv => '$val * 1000',
         PrintConv => '"$val m"',
-        PrintConvInv => '$val=~s/ .*//;$val',
+        PrintConvInv => '$val=~s/\s*m$//;$val',
     },
     0x0007 => {
         Name => 'WhiteBalance',
@@ -111,6 +111,7 @@ $VERSION = '1.17';
             129 => 'Manual',
         },
     },
+    # 0x0009 Bulb? (ref unknown)
     0x000a => {
         Name => 'DigitalZoom',
         Writable => 'int32u',
@@ -258,6 +259,7 @@ $VERSION = '1.17';
         Writable => 'int16u',
         Count => 2,
         PrintConv => '$val =~ tr/ /x/; $val',
+        PrintConvInv => '$val =~ tr/x/ /; $val',
     },
     0x0003 => {
         Name => 'PreviewImageLength',
@@ -429,7 +431,7 @@ $VERSION = '1.17';
         ValueConv => '$val / 1000',
         ValueConvInv => '$val * 1000',
         PrintConv => '"$val m"',
-        PrintConvInv => '$val=~s/ .*//;$val',
+        PrintConvInv => '$val=~s/\s*m$//;$val',
     },
     # 0x2023 looks interesting (values 0,1,2,3,5 in samples) - PH
     0x2034 => {
@@ -565,7 +567,7 @@ Casio maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
