@@ -27,6 +27,9 @@
 #              13) Roger Larsson private communication (tests with D200)
 #              14) http://homepage3.nifty.com/kamisaka/makernote/makernote_nikon.htm
 #              15) http://tomtia.plala.jp/DigitalCamera/MakerNote/index.asp
+#              16) Jeffrey Friedl private communication (D200 with firmware update)
+#              17) http://www.wohlberg.net/public/software/photo/nstiffexif/
+#              18) Anonymous user private communication (D70, D200, D2x)
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::Nikon;
@@ -36,7 +39,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.45';
+$VERSION = '1.69';
 
 # nikon lens ID numbers (ref 8/11)
 my %nikonLensIDs = (
@@ -48,6 +51,8 @@ my %nikonLensIDs = (
     },
     # (hex digits must be uppercase in keys below)
     '01 58 50 50 14 14 02 00' => 'AF Nikkor 50mm f/1.8',
+    '01 00 00 00 00 00 02 00' => 'AF Teleconverter TC-16A 1.6x',
+    '01 00 00 00 00 00 08 00' => 'AF Teleconverter TC-16A 1.6x',
     '02 42 44 5C 2A 34 02 00' => 'AF Zoom-Nikkor 35-70mm f/3.3-4.5',
     '02 42 44 5C 2A 34 08 00' => 'AF Zoom-Nikkor 35-70mm f/3.3-4.5',
     '03 48 5C 81 30 30 02 00' => 'AF Zoom-Nikkor 70-210mm f/4',
@@ -75,6 +80,7 @@ my %nikonLensIDs = (
     '1C 48 30 30 24 24 12 00' => 'AF Nikkor 20mm f/2.8',
     '1D 42 44 5C 2A 34 12 00' => 'AF Zoom-Nikkor 35-70mm f/3.3-4.5 N',
     '1E 54 56 56 24 24 13 00' => 'AF Micro-Nikkor 60mm f/2.8',
+    '1F 54 6A 6A 24 24 14 00' => 'AF Micro-Nikkor 105mm f/2.8',
     '20 48 60 80 24 24 15 00' => 'AF Zoom-Nikkor ED 80-200mm f/2.8',
     '22 48 72 72 18 18 16 00' => 'AF DC-Nikkor 135mm f/2',
     '24 48 60 80 24 24 1A 02' => 'AF Zoom-Nikkor ED 80-200mm f/2.8D',
@@ -149,8 +155,12 @@ my %nikonLensIDs = (
     '8A 54 6A 6A 24 24 8C 0E' => 'AF-S VR Micro-Nikkor 105mm f/2.8G IF-ED', #10
     '8B 40 2D 80 2C 3C 8D 0E' => 'AF-S DX VR Zoom-Nikkor 18-200mm f/3.5-5.6G IF-ED',
     '8C 40 2D 53 2C 3C 8E 06' => 'AF-S DX Zoom-Nikkor 18-55mm f/3.5-5.6G ED',
+    '8D 44 5C 8E 34 3C 8F 0E' => 'AF-S VR Zoom-Nikkor 70-300mm f/4.5-5.6G IF-ED', #10
+    '8F 40 2D 72 2C 3C 91 06' => 'AF-S DX Zoom-Nikkor 18-135mm f/3.5-5.6G IF-ED',
+    '90 3B 53 80 30 3C 92 0E' => 'AF-S DX VR Zoom-Nikkor 55-200mm f/4-5.6G IF-ED',
+    '94 40 2D 53 2C 3C 96 06' => 'AF-S DX Zoom-Nikkor 18-55mm f/3.5-5.6G ED II', #10 (D40)
 #
-    '06 3F 68 68 2C 2C 06 00' => 'Cosina 100mm F/3.5 Macro',
+    '06 3F 68 68 2C 2C 06 00' => 'Cosina 100mm f/3.5 Macro',
 #
     '26 48 11 11 30 30 1C 02' => 'Sigma 8mm F4 EX Circular Fisheye',
     '48 3C 19 31 30 3C 4B 06' => 'Sigma 10-20mm F4-5.6 EX DC HSM',
@@ -161,6 +171,7 @@ my %nikonLensIDs = (
     '26 40 27 3F 2C 34 1C 02' => 'Sigma 15-30mm F3.5-4.5 EX Aspherical DG DF',
     '48 48 2B 44 24 30 4B 06' => 'Sigma 17-35mm F2.8-4 EX DG  Aspherical HSM',
     '26 54 2B 44 24 30 1C 02' => 'Sigma 17-35mm F2.8-4 EX ASPHERICAL',
+    '7F 48 2B 5C 24 34 1C 06' => 'Sigma 17-70mm F2.8-4.5 DC MACRO Asp. IF',
     '26 48 2D 50 24 24 1C 06' => 'Sigma 18-50mm F2.8 EX DC',
     '26 40 2D 50 2C 3C 1C 06' => 'Sigma 18-50mm F3.5-5.6 DC',
     '26 40 2D 70 2B 3C 1C 06' => 'Sigma 18-125mm F3.5-5.6 DC',
@@ -170,6 +181,7 @@ my %nikonLensIDs = (
     '02 46 37 37 25 25 02 00' => 'Sigma 24mm F2.8 Macro',
     '26 48 37 56 24 24 1C 02' => 'Sigma 24-60mm F2.8 EX DG',
     '26 54 37 5C 24 24 1C 02' => 'Sigma 24-70mm F2.8 EX DG Macro',
+    '67 54 37 5C 24 24 1C 02' => 'Sigma 24-70mm F2.8 EX DG Macro',
     '26 40 37 5C 2C 3C 1C 02' => 'Sigma 24-70mm F3.5-5.6 ASPHERICAL HF',
     '26 54 37 73 24 34 1C 02' => 'Sigma 24-135mm F2.8-4.5',
     '26 58 3C 3C 14 14 1C 02' => 'Sigma 28mm F1.8 EX DG DF',
@@ -186,61 +198,72 @@ my %nikonLensIDs = (
     '48 54 3E 3E 0C 0C 4B 06' => 'Sigma 30mm F1.4 EX DC HSM',
     '02 40 44 73 2B 36 02 00' => 'Sigma 35-135mm F3.5-4.5 a',
     '32 54 50 50 24 24 35 02' => 'Sigma 50mm F2.8 EX DG Macro',
-    '48 3C 50 A0 30 40 4B 02' => 'Sigma 50-500mmF4-6.3 EX APO RF HSM',
+    '7A 47 50 76 24 24 4B 06' => 'Sigma APO 50-150mm F2.8 EX DC HSM',
+    '48 3C 50 A0 30 40 4B 02' => 'Sigma 50-500mm F4-6.3 EX APO RF HSM',
     '26 3C 54 80 30 3C 1C 06' => 'Sigma 55-200mm F4-5.6 DC',
     '48 54 5C 80 24 24 4B 02' => 'Sigma 70-200mm F2.8 EX APO IF HSM',
+    '26 3C 5C 82 30 3C 1C 02' => 'Sigma 70-210mm F4-5.6 UC-II',
     '26 3C 5C 8E 30 3C 1C 02' => 'Sigma 70-300mm F4-5.6 DG MACRO',
     '56 3C 5C 8E 30 3C 1C 02' => 'Sigma 70-300mm F4-5.6 APO Macro Super II',
     '02 37 5E 8E 35 3D 02 00' => 'Sigma 75-300mm F4.5-5.6 APO',
     '02 48 65 65 24 24 02 00' => 'Sigma 90mm F2.8 Macro',
     '77 44 61 98 34 3C 7B 0E' => 'Sigma 80-400mm f4.5-5.6 EX OS',
     '48 48 68 8E 30 30 4B 02' => 'Sigma 100-300mm F4 EX IF HSM',
+    '48 54 6F 8E 24 24 4B 02' => 'Sigma APO 120-300mm F2.8 EX DG HSM',
     '26 44 73 98 34 3C 1C 02' => 'Sigma 135-400mm F4.5-5.6 APO Aspherical',
     '48 48 76 76 24 24 4B 06' => 'Sigma 150mm F2.8 EX DG APO Macro HSM',
     '26 40 7B A0 34 40 1C 02' => 'Sigma APO 170-500mm F5-6.3 ASPHERICAL RF',
     '48 4C 7D 7D 2C 2C 4B 02' => 'Sigma APO MACRO 180mm F3.5 EX DG HSM',
     '48 54 8E 8E 24 24 4B 02' => 'Sigma APO 300mm F2.8 EX DG HSM',
     '26 48 8E 8E 30 30 1C 02' => 'Sigma APO TELE MACRO 300mm F4',
+    '48 3C 8E B0 3C 3C 4B 02' => 'Sigma APO 300-800 F5.6 EX DG HSM',
     '02 2F 98 98 3D 3D 02 00' => 'Sigma 400mm F5.6 APO',
 #
     '03 43 5C 81 35 35 02 00' => 'Soligor AF C/D ZOOM UMCS 70-210mm 1:4.5',
 #
-    '00 3C 1F 37 30 30 00 06' => 'Tokina AT-X 124 AF PRO DX - AF 12-24mm f/4',
-    '00 40 2B 2B 2C 2C 00 02' => 'Tokina AT-X 17 AF PRO - AF 17mm f/3.5',
-    '25 48 3C 5C 24 24 1B 02' => 'Tokina AT-X287AF PRO SV 28-70mm F2.8',
-    '00 48 3C 60 24 24 00 02' => 'Tokina AT-X280AF PRO 28-80mm F2.8 ASPHERICAL',
-    '14 54 60 80 24 24 0B 00' => 'Tokina AT-X828AF 80-200mm F2.8',
-    '24 44 60 98 34 3C 1A 02' => 'Tokina AT-X840AF II 80-400mm F4.5-5.6',
+    '00 40 18 2B 2C 34 00 06' => 'Tokina AT-X 107 DX Fish-Eye - AF 10-17mm F3.5-4.5',
+    '00 3C 1F 37 30 30 00 06' => 'Tokina AT-X 124 AF PRO DX - AF 12-24mm F4',
+    '00 48 29 50 24 24 00 06' => 'Tokina AT-X 165 PRO DX - AF 16-50mm F2.8',
+    '00 40 2B 2B 2C 2C 00 02' => 'Tokina AT-X 17 AF PRO - AF 17mm F3.5',
+    '25 48 3C 5C 24 24 1B 02' => 'Tokina AT-X 287 AF PRO SV 28-70mm F2.8',
+    '00 48 3C 60 24 24 00 02' => 'Tokina AT-X 280 AF PRO 28-80mm F2.8 ASPHERICAL',
+    '00 48 50 72 24 24 00 06' => 'Tokina AT-X 535 PRO DX - AF 50-135mm F2.8',
+    '14 54 60 80 24 24 0B 00' => 'Tokina AT-X 828 AF 80-200mm F2.8',
+    '24 44 60 98 34 3C 1A 02' => 'Tokina AT-X 840 AF II 80-400mm F4.5-5.6',
+    '00 44 60 98 34 3C 00 02' => 'Tokina AT-X 840D 80-400mm F4.5-5.6', #PH
     '00 54 68 68 24 24 00 02' => 'Tokina AT-X M100 PRO D - 100mm F2.8',
-    '14 48 68 8E 30 30 0B 00' => 'Tokina AT-X340AF II 100-300mm F4',
-    '00 54 8E 8E 24 24 00 02' => 'Tokina AT-X300AF PRO 300mm F2.8',
+    '14 48 68 8E 30 30 0B 00' => 'Tokina AT-X 340 AF II 100-300mm F4',
+    '00 54 8E 8E 24 24 00 02' => 'Tokina AT-X 300 AF PRO 300mm F2.8',
 #
-    '00 36 1C 2D 34 3C 00 06' => 'Tamron SP AF11-18mm F/4.5-5.6 Di II LD Aspherical (IF)',
-    '07 46 2B 44 24 30 03 02' => 'Tamron SP AF17-35mm F/2.8-4 Di LD Aspherical (IF)',
+    '00 36 1C 2D 34 3C 00 06' => 'Tamron SP AF11-18mm f/4.5-5.6 Di II LD Aspherical (IF)',
+    '07 46 2B 44 24 30 03 02' => 'Tamron SP AF17-35mm f/2.8-4 Di LD Aspherical (IF)',
+    '00 53 2B 50 24 24 00 06' => 'Tamron SP AF17-50mm f/2.8 (A16)', #PH
     '00 3F 2D 80 2B 40 00 06' => 'Tamron AF18-200mm f/3.5-6.3 XR Di II LD Aspherical (IF)',
-    '00 3F 2D 80 2C 40 00 06' => 'Tamron AF18-200mm f/3.5-6.3 XR Di II LD Aspherical (IF) MACRO',
-    '07 40 30 45 2D 35 03 02' => 'Tamron AF19-35mm F3.5-4.5',
+    '00 3F 2D 80 2C 40 00 06' => 'Tamron AF18-200mm f/3.5-6.3 XR Di II LD Aspherical (IF) Macro',
+    '07 40 2F 44 2C 34 03 02' => 'Tamron AF19-35mm f/3.5-4.5 N',
+    '07 40 30 45 2D 35 03 02' => 'Tamron AF19-35mm f/3.5-4.5',
     '00 49 30 48 22 2B 00 02' => 'Tamron SP AF20-40mm f/2.7-3.5',
-    '45 41 37 72 2C 3C 48 02' => 'Tamron SP AF24-135mm F3.5-5.6 AD ASPHERICAL(IF)MACRO',
-    '33 54 3C 5E 24 24 62 02' => 'Tamron SP AF28-75mm F2.8 XR Di LD ASPHERICAL(IF)MACRO',
+    '0E 4A 31 48 23 2D 0E 02' => 'Tamron SP AF20-40mm f/2.7-3.5',
+    '45 41 37 72 2C 3C 48 02' => 'Tamron SP AF24-135mm f/3.5-5.6 AD Aspherical (IF) Macro',
+    '33 54 3C 5E 24 24 62 02' => 'Tamron SP AF28-75mm f/2.8 XR Di LD Aspherical (IF) Macro',
     '10 3D 3C 60 2C 3C D2 02' => 'Tamron AF28-80mm f/3.5-5.6 Aspherical',
-    '45 3D 3C 60 2C 3C 48 02' => 'Tamron AF 28-80mm F3.5-5.6 ASPHERICAL',
+    '45 3D 3C 60 2C 3C 48 02' => 'Tamron AF28-80mm f/3.5-5.6 Aspherical',
     '00 48 3C 6A 24 24 00 02' => 'Tamron SP AF28-105mm f/2.8',
     '0B 3E 3D 7F 2F 3D 0E 02' => 'Tamron AF28-200mm f/3.8-5.6D',
     '0B 3E 3D 7F 2F 3D 0E 00' => 'Tamron AF28-200mm f/3.8-5.6',
     '4D 41 3C 8E 2B 40 62 02' => 'Tamron AF28-300mm f/3.5-6.3 XR Di LD Aspherical (IF)',
     '4D 41 3C 8E 2C 40 62 02' => 'Tamron AF28-300mm f/3.5-6.3D',
-    '69 48 5C 8E 30 3C 6F 02' => 'Tamron AF70-300mm F4-5.6 LD MACRO 1:2',
+    '69 48 5C 8E 30 3C 6F 02' => 'Tamron AF70-300mm f/4-5.6 LD Macro 1:2',
     '32 53 64 64 24 24 35 02' => 'Tamron SP AF90mm f/2.8 Di 1:1 Macro',
-    '00 4C 7C 7C 2C 2C 00 02' => 'Tamron SP AF180mm F3.5 Di Model B01',
+    '00 4C 7C 7C 2C 2C 00 02' => 'Tamron SP AF180mm f/3.5 Di Model B01',
     '20 3C 80 98 3D 3D 1E 02' => 'Tamron AF200-400mm f/5.6 LD IF',
     '00 3E 80 A0 38 3F 00 02' => 'Tamron SP AF200-500mm f/5-6.3 Di LD (IF)',
-    '00 3F 80 A0 38 3F 00 02' => 'Tamron SP AF200-500mm F5-6.3 Di',
+    '00 3F 80 A0 38 3F 00 02' => 'Tamron SP AF200-500mm f/5-6.3 Di',
 #
     '00 00 00 00 00 00 00 01' => 'Manual Lens No CPU',
-    '1E 5D 64 64 20 20 13 00' => 'Unknown 90mm F/2.5',
-    '2F 40 30 44 2C 34 29 02' => 'Unknown 20-35mm F/3.5-4.5D',
-    '12 3B 68 8D 3D 43 09 02' => 'Unknown 100-290mm F5.6-6.7',
+    '1E 5D 64 64 20 20 13 00' => 'Unknown 90mm f/2.5',
+    '2F 40 30 44 2C 34 29 02' => 'Unknown 20-35mm f/3.5-4.5D',
+    '12 3B 68 8D 3D 43 09 02' => 'Unknown 100-290mm f/5.6-6.7',
 );
 
 # Nikon maker note tags
@@ -254,7 +277,7 @@ my %nikonLensIDs = (
     0x0001 => { #2
         # the format differs for different models.  for D70, this is a string '0210',
         # but for the E775 it is binary: "\x00\x01\x00\x00"
-        Name => 'FirmwareVersion',
+        Name => 'MakerNoteVersion',
         Writable => 'undef',
         Count => 4,
         # convert to string if binary
@@ -268,11 +291,12 @@ my %nikonLensIDs = (
         # (may be different than ISO setting if auto)
         Name => 'ISO',
         Writable => 'int16u',
-        Priority => 0,  # the EXIF ISO is more reliable
         Count => 2,
+        Priority => 0,  # the EXIF ISO is more reliable
         Groups => { 2 => 'Image' },
-        PrintConv => '$_=$val;s/^0 //;$_',
-        PrintConvInv => '"0 $val"',
+        # first number is 1 for "Hi ISO" modes (H0.3, H0.7 and H1.0 on D80) - PH
+        PrintConv => '$_=$val;s/^0 //;s/^1 (\d+)/Hi $1/;$_',
+        PrintConvInv => '$_=$val;/^\d+/ ? "0 $_" : (s/Hi ?//i ? "1 $_" : $_)',
     },
     0x0003 => { Name => 'ColorMode',    Writable => 'string' },
     0x0004 => { Name => 'Quality',      Writable => 'string' },
@@ -287,7 +311,7 @@ my %nikonLensIDs = (
         Writable => 'string',
         Count => 13,
     },
-    0x000b => { Name => 'WhiteBalanceFineTune', Writable => 'int16u' }, #2
+    0x000b => { Name => 'WhiteBalanceFineTune', Writable => 'int16s' }, #2
     0x000c => {
         Name => 'ColorBalance1',
         Writable => 'rational64u',
@@ -321,7 +345,7 @@ my %nikonLensIDs = (
     0x0010 => {
         Name => 'DataDump',
         Writable => 0,
-        ValueConv => '\$val',
+        Binary => 1,
     },
     0x0011 => {
         Name => 'NikonPreview',
@@ -373,16 +397,18 @@ my %nikonLensIDs = (
     },
     0x0019 => { #5
         Name => 'ExposureBracketValue',
-        Format => 'rational64s',
+        Writable => 'rational64s',
         PrintConv => 'Image::ExifTool::Exif::ConvertFraction($val)',
         PrintConvInv => 'eval $val',
     },
     0x001a => { #PH
         Name => 'ImageProcessing',
-        Format => 'string',
+        Writable => 'string',
     },
     0x001b => { #15
         Name => 'CropHiSpeed',
+        Writable => 'int16u',
+        Count => 7,
         PrintConv => q{
             my @a = split ' ', $val;
             return "Unknown ($val)" unless @a == 7;
@@ -401,9 +427,14 @@ my %nikonLensIDs = (
         Writable => 'int16u',
         PrintConv => {
             1 => 'sRGB',
-            2 => 'AdobeRGB',
+            2 => 'Adobe RGB',
         },
     },
+    0x0020 => { #16
+        Name => 'ImageAuthentication',
+        Writable => 'int8u',
+        PrintConv => { 0 => 'Off', 1 => 'On' },
+    },
     0x0080 => { Name => 'ImageAdjustment',  Writable => 'string' },
     0x0081 => { Name => 'ToneComp',         Writable => 'string' }, #2
     0x0082 => { Name => 'AuxiliaryLens',    Writable => 'string' },
@@ -496,6 +527,7 @@ my %nikonLensIDs = (
     },
     0x008a => { #15
         Name => 'AutoBracketRelease',
+        Writable => 'int16u',
         PrintConv => {
             0 => 'None',
             1 => 'Auto Release',
@@ -514,7 +546,7 @@ my %nikonLensIDs = (
     0x008c => {
         Name => 'NEFCurve1',
         Writable => 0,
-        ValueConv => '\$val',
+        Binary => 1,
     },
     0x008d => { Name => 'ColorHue' ,        Writable => 'string' }, #2
     # SceneMode takes on the following values: PORTRAIT, PARTY/INDOOR, NIGHT PORTRAIT,
@@ -524,6 +556,28 @@ my %nikonLensIDs = (
     # LightSource shows 3 values COLORED SPEEDLIGHT NATURAL.
     # (SPEEDLIGHT when flash goes. Have no idea about difference between other two.)
     0x0090 => { Name => 'LightSource',      Writable => 'string' }, #2
+    0x0091 => [ #18
+        {
+            Condition => '$$valPt =~ /^02/',
+            Name => 'ShotInfo',
+            Writable => 0, # can't yet write encrypted data
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Nikon::ShotInfo',
+                ProcessProc => \&Image::ExifTool::Nikon::ProcessNikonEncrypted,
+                DecryptStart => 4,
+                DecryptLen => 0x90,
+            },
+        },
+        {
+            Name => 'ShotInfo',
+            Writable => 0,
+            Unknown => 1, # no tags known so don't process unless necessary
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Nikon::ShotInfo',
+                DirOffset => 4,
+            },
+        },
+    ],
     0x0092 => { #2
         Name => 'HueAdjustment',
         Writable => 'int16s',
@@ -538,13 +592,12 @@ my %nikonLensIDs = (
     0x0096 => {
         Name => 'NEFCurve2',
         Writable => 0,
-        ValueConv => '\$val',
+        Binary => 1,
     },
     0x0097 => [ #4
         {
             Condition => '$$valPt =~ /^0100/', # (D100)
             Name => 'ColorBalance0100',
-            Writable => 0,
             SubDirectory => {
                 Start => '$valuePtr + 72',
                 TagTable => 'Image::ExifTool::Nikon::ColorBalance1',
@@ -553,7 +606,6 @@ my %nikonLensIDs = (
         {
             Condition => '$$valPt =~ /^0102/', # (D2H)
             Name => 'ColorBalance0102',
-            Writable => 0,
             SubDirectory => {
                 Start => '$valuePtr + 10',
                 TagTable => 'Image::ExifTool::Nikon::ColorBalance2',
@@ -562,7 +614,6 @@ my %nikonLensIDs = (
         {
             Condition => '$$valPt =~ /^0103/', # (D70)
             Name => 'ColorBalance0103',
-            Writable => 0,
             # D70:  at file offset 'tag-value + base + 20', 4 16 bits numbers,
             # v[0]/v[1] , v[2]/v[3] are the red/blue multipliers.
             SubDirectory => {
@@ -573,19 +624,19 @@ my %nikonLensIDs = (
         {
             Condition => '$$valPt =~ /^0205/', # (D50)
             Name => 'ColorBalance0205',
-            Writable => 0,
+            Writable => 0, # can't yet write encrypted data
             SubDirectory => {
                 TagTable => 'Image::ExifTool::Nikon::ColorBalance2',
                 ProcessProc => \&Image::ExifTool::Nikon::ProcessNikonEncrypted,
                 DecryptStart => 4,
-                DecryptLen => 22, # 324 bytes encrypted, but don't need to decrypt it all
+                DecryptLen => 22, # 284 bytes encrypted, but don't need to decrypt it all
                 DirOffset => 14,
             },
         },
         {
-            Condition => '$$valPt =~ /^02/', # (0204=D2X,0206=D2Hs,0207=D200)
+            Condition => '$$valPt =~ /^02/', # (D2X=0204,D2Hs=0206,D200=0207,D40=0208)
             Name => 'ColorBalance02',
-            Writable => 0,
+            Writable => 0, # can't yet write encrypted data
             SubDirectory => {
                 TagTable => 'Image::ExifTool::Nikon::ColorBalance2',
                 ProcessProc => \&Image::ExifTool::Nikon::ProcessNikonEncrypted,
@@ -616,8 +667,9 @@ my %nikonLensIDs = (
         },
         # note: this information is encrypted if the version is 02xx
         { #8
-            Condition => '$$valPt =~ /^0201/',
+            Condition => '$$valPt =~ /^020(1|2)/', # D80/D40 are version 0202 (PH)
             Name => 'LensData0201',
+            Writable => 0, # can't yet write encrypted data
             SubDirectory => {
                 TagTable => 'Image::ExifTool::Nikon::LensData01',
                 ProcessProc => \&Image::ExifTool::Nikon::ProcessNikonEncrypted,
@@ -641,8 +693,12 @@ my %nikonLensIDs = (
         PrintConv => '$val=~s/ / x /;"$val um"',
         PrintConvInv => '$val=~tr/a-zA-Z/ /;$val',
     },
+    # 0x009c - observed values: "", "TWO-SHOT" - PH
     0x00a0 => { Name => 'SerialNumber',     Writable => 'string' }, #2
-    0x00a2 => { Name => 'ImageDataSize' }, # size of compressed image data plus EOI segment (ref 10)
+    0x00a2 => { # size of compressed image data plus EOI segment (ref 10)
+        Name => 'ImageDataSize',
+        Writable => 'int32u',
+    },
     0x00a5 => { #15
         Name => 'ImageCount',
         Writable => 'int32u',
@@ -667,8 +723,16 @@ my %nikonLensIDs = (
     0x00ab => { Name => 'VariProgram',      Writable => 'string' }, #2
     0x00ac => { Name => 'ImageStabilization',Writable=> 'string' }, #14
     0x00ad => { Name => 'AFResponse',       Writable => 'string' }, #14
+    0x00b0 => { #PH
+        Name => 'MultiExposure',
+        Condition => '$$valPt =~ /^0100/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Nikon::MultiExposure',
+        },
+    },
     0x00b1 => { #14
         Name => 'HighISONoiseReduction',
+        Writable => 'int16u',
         PrintConv => {
             0 => 'Off',
             1 => 'On for ISO 1600/3200',
@@ -677,6 +741,7 @@ my %nikonLensIDs = (
             6 => 'Strong',
         },
     },
+    # 0x00b2 (string: 'Normal', 0xc3's, 0xff's or 0x20's)
     0x0e00 => {
         Name => 'PrintIM',
         Description => 'Print Image Matching',
@@ -691,26 +756,102 @@ my %nikonLensIDs = (
     # utility that blindly copies the maker notes (not ExifTool) - PH
     0x0e01 => {
         Name => 'NikonCaptureData',
-        Writable => 0,
         SubDirectory => {
             TagTable => 'Image::ExifTool::NikonCapture::Main',
         },
     },
-    0x0e09 => 'NikonCaptureVersion', #12
+    0x0e09 => { #12
+        Name => 'NikonCaptureVersion',
+        Writable => 'string',
+    },
     # 0x0e0e is in D70 Nikon Capture files (not out-of-the-camera D70 files) - PH
     0x0e0e => { #PH
         Name => 'NikonCaptureOffsets',
-        Writable => 0,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Nikon::CaptureOffsets',
             Validate => '$val =~ /^0100/',
             Start => '$valuePtr + 4',
         },
     },
-    # 0x0e10 - unknown (written by Nikon Capture)
+    0x0e10 => { #17
+        Name => 'NikonScanIFD',
+        Groups => { 1 => 'NikonScan', 2 => 'Image' },
+        Flags => 'SubIFD',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Nikon::Scan',
+            Start => '$val',
+        },
+    },
     # 0x0e13 - some sort of edit history written by Nikon Capture
 );
 
+# NikonScan IFD entries (ref 17)
+%Image::ExifTool::Nikon::Scan = (
+    WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
+    CHECK_PROC => \&Image::ExifTool::Exif::CheckExif,
+    WRITE_GROUP => 'NikonScan',
+    WRITABLE => 1,
+    GROUPS => { 0 => 'MakerNotes', 1 => 'NikonScan', 2 => 'Image' },
+    NOTES => 'This information is written by the Nikon Scan software.',
+    0x02 => { Name => 'FilmType',    Writable => 'string', },
+    0x40 => { Name => 'MultiSample', Writable => 'string' },
+    0x41 => { Name => 'BitDepth',    Writable => 'int16u' },
+    0x50 => {
+        Name => 'MasterGain',
+        Writable => 'rational64s',
+        PrintConv => 'sprintf("%.2f",$val)',
+        PrintConvInv => '$val',
+    },
+    0x51 => {
+        Name => 'ColorGain',
+        Writable => 'rational64s',
+        Count => 3,
+        PrintConv => 'sprintf("%.2f %.2f %.2f",split(" ",$val))',
+        PrintConvInv => '$val',
+    },
+    0x100 => { Name => 'DigitalICE', Writable => 'string' },
+    0x110 => {
+        Name => 'ROCInfo',
+        SubDirectory => { TagTable => 'Image::ExifTool::Nikon::ROC' },
+    },
+    0x120 => {
+        Name => 'GEMInfo',
+        SubDirectory => { TagTable => 'Image::ExifTool::Nikon::GEM' },
+    },
+);
+
+# ref 17
+%Image::ExifTool::Nikon::ROC = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    FORMAT => 'int32u',
+    WRITABLE => 1,
+    FIRST_ENTRY => 0,
+    GROUPS => { 0 => 'MakerNotes', 1 => 'NikonScan', 2 => 'Image' },
+    0 => {
+        Name => 'DigitalROC',
+        ValueConv => '$val / 10',
+        ValueConvInv => 'int($val * 10)',
+    },
+);
+
+# ref 17
+%Image::ExifTool::Nikon::GEM = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    FORMAT => 'int32u',
+    WRITABLE => 1,
+    FIRST_ENTRY => 0,
+    GROUPS => { 0 => 'MakerNotes', 1 => 'NikonScan', 2 => 'Image' },
+    0 => {
+        Name => 'DigitalGEM',
+        ValueConv => '$val<95 ? $val/20-1 : 4',
+        ValueConvInv => '$val == 4 ? 95 : int(($val + 1) * 20)',
+    },
+);
+
 # Nikon AF information (ref 13)
 %Image::ExifTool::Nikon::AFInfo = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
@@ -720,7 +861,7 @@ my %nikonLensIDs = (
     FIRST_ENTRY => 0,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
     0 => {
-        Name => 'AFMode',
+        Name => 'AFAreaMode',
         PrintConv => {
             0 => 'Single Area',
             1 => 'Dynamic Area',
@@ -801,6 +942,10 @@ my %nikonLensIDs = (
     WRITABLE => 1,
     FORMAT => 'int16u',
     FIRST_ENTRY => 0,
+    NOTES => q{
+        This information is encrypted for most camera models, and if encrypted is
+        not currently writable by exiftool.
+    },
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
     0 => {
         Name => 'WB_RGGBLevels',
@@ -830,10 +975,7 @@ my %nikonLensIDs = (
     CHECK_PROC => \&Image::ExifTool::Exif::CheckExif,
     WRITABLE => 1,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
-    0x0003 => {
-        Name => 'Quality',
-        Description => 'Image Quality',
-    },
+    0x0003 => 'Quality',
     0x0004 => 'ColorMode',
     0x0005 => 'ImageAdjustment',
     0x0006 => 'CCDSensitivity',
@@ -957,20 +1099,27 @@ my %nikonFocalConversions = (
     0x0c => 'MCUVersion', #8
 );
 
-# Nikon lens data (note: needs decrypting if LensDataVersion is 0201)
+# Nikon lens data (note: needs decrypting if LensDataVersion is 020x)
 %Image::ExifTool::Nikon::LensData01 = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     FIRST_ENTRY => 0,
     NOTES => q{
-        Nikon encrypts the LensData information below if LensDataVersion is 0201,
-        but  the decryption algorithm is known so the information can be extracted.
-        It isn't yet writable, however, because the encryption adds complications
-        which make writing more difficult.
+        Nikon encrypts the LensData information below if LensDataVersion is 0201 or
+        higher, but  the decryption algorithm is known so the information can be
+        extracted.  It isn't yet writable, however, because the encryption adds
+        complications which make writing more difficult.
     },
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
     0x00 => {
         Name => 'LensDataVersion',
-        Format => 'undef[4]',
+        Format => 'string[4]',
+    },
+    0x04 => { #8
+        Name => 'ExitPupilPosition',
+        ValueConv => '$val ? 2048 / $val : $val',
+        ValueConvInv => '$val ? 2048 / $val : $val',
+        PrintConv => 'sprintf("%.1fmm",$val)',
+        PrintConvInv => '$val=~s/ ?mm$//; $val',
     },
     0x05 => { #8
         Name => 'AFAperture',
@@ -985,11 +1134,12 @@ my %nikonFocalConversions = (
         # With older AF lenses this does not work... (ref 13)
         # ie) AF Nikkor 50mm f/1.4 => 48 (0x30)
         # AF Zoom-Nikkor 35-105mm f/3.5-4.5 => @35mm => 15 (0x0f), @105mm => 141 (0x8d)
+        Notes => 'this focus distance is approximate, and not very accurate for some lenses',
         Name => 'FocusDistance',
         ValueConv => '0.01 * 10**($val/40)', # in m
         ValueConvInv => '$val>0 ? 40*log($val*100)/log(10) : 0',
         PrintConv => '$val ? sprintf("%.2f m",$val) : "inf"',
-        PrintConvInv => '$val eq "inf" ? 0 : $val =~ s/\s.*//, $val',
+        PrintConvInv => '$val eq "inf" ? 0 : $val =~ s/\s*m$//, $val',
     },
     0x0a => { #8/9
         Name => 'FocalLength',
@@ -1030,6 +1180,84 @@ my %nikonFocalConversions = (
     },
 );
 
+# information encrypted in some cameras - ref 18
+%Image::ExifTool::Nikon::ShotInfo = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    FIRST_ENTRY => 0,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    DATAMEMBER => [ 0 ],
+    NOTES => 'This information is encrypted for ShotInfoVersion 02xx.',
+    0x00 => {
+        Name => 'ShotInfoVersion',
+        RawConv => '$$self{ShotInfoVersion} = $val',
+        Format => 'string[4]',
+    },
+    0x66 => {
+        Name => 'VR_0x66',
+        Condition => '$$self{ShotInfoVersion} =~ /^(0204)$/',
+        Format => 'int8u',
+        Unknown => 1,
+        PrintConv => {
+            0 => 'VR off',
+            1 => 'VR on (normal)',
+            2 => 'VR on (active)',
+        },
+    },
+    # 6a, 6e not correct for 0103 (D70), 0207 (D200)
+    0x6a => {
+        Name => 'ShutterCount',
+        Condition => '$$self{ShotInfoVersion} =~ /^(0204)$/',
+        Format => 'int32u',
+        Priority => 0,
+    },
+    0x6e => {
+        Name => 'DeletedImageCount',
+        Condition => '$$self{ShotInfoVersion} =~ /^(0204)$/',
+        Format => 'int32u',
+        Priority => 0,
+    },
+    0x82 => {
+        Name => 'VR_0x82',
+        Condition => '$$self{ShotInfoVersion} =~ /^(0204)$/',
+        Format => 'int8u',
+        Unknown => 1,
+        PrintConv => {
+            0 => 'VR off',
+            1 => 'VR on (normal)',
+            2 => 'code 2 (active?)',
+        },
+    },
+    # note: DecryptLen currently set to 0x90
+);
+
+# Multi exposure / image overlay information (ref PH)
+%Image::ExifTool::Nikon::MultiExposure = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    WRITABLE => 1,
+    FIRST_ENTRY => 0,
+    FORMAT => 'int32u',
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    0 => {
+        Name => 'MultiExposureVersion',
+        Format => 'string[4]',
+    },
+    1 => {
+        Name => 'MultiExposureMode',
+        PrintConv => {
+            0 => 'Off',
+            1 => 'Multiple Exposure',
+            2 => 'Image Overlay',
+        },
+    },
+    2 => 'MultiExposureShots',
+    3 => {
+        Name => 'MultiExposureAutoGain',
+        PrintConv => { 0 => 'Off', 1 => 'On' },
+    },
+);
+
 # tags in Nikon QuickTime videos (PH - observations with Coolpix S3)
 # (note: very similar to information in Pentax videos)
 %Image::ExifTool::Nikon::MOV = (
@@ -1046,6 +1274,7 @@ my %nikonFocalConversions = (
     },
     0x18 => {
         Name => 'Model',
+        Description => 'Camera Model Name',
         Format => 'string[8]',
     },
     0x26 => {
@@ -1108,7 +1337,7 @@ my %nikonFocalConversions = (
         PrintConv => '"$prt[0] $prt[1]"',
     },
     LensID => {
-        SeparateTable => 'Nikon',   # print values in a separate table
+        SeparateTable => 'Nikon LensID',    # print values in a separate table
         Require => {
             0 => 'Nikon:LensIDNumber',
             1 => 'LensFStops',
@@ -1126,7 +1355,7 @@ my %nikonFocalConversions = (
 );
 
 # add our composite tags
-Image::ExifTool::AddCompositeTags('Image::ExifTool::Nikon::Composite');
+Image::ExifTool::AddCompositeTags('Image::ExifTool::Nikon');
 
 
 #------------------------------------------------------------------------------
@@ -1296,7 +1525,10 @@ sub ProcessNikon($$$)
             my ($start, $len, $offset);
             if ($tagInfo and $$tagInfo{SubDirectory}) {
                 $start = $tagInfo->{SubDirectory}->{DecryptStart};
-                $len = $tagInfo->{SubDirectory}->{DecryptLen};
+                # may decrypt only part of the information to save time
+                if ($verbose < 3 and $exifTool->Options('Unknown') < 2) {
+                    $len = $tagInfo->{SubDirectory}->{DecryptLen};
+                }
                 $offset = $tagInfo->{SubDirectory}->{DirOffset};
             }
             $start or $start = 0;
@@ -1314,17 +1546,17 @@ sub ProcessNikon($$$)
             }
             # use fixed serial numbers if no good serial number found
             unless ($serial =~ /^\d+$/) {
-                if ($exifTool->{CameraModel} =~ /\bD200$/) {
-                    $serial = 0x60; # D200 (ref 10)
-                } else {
+                if ($exifTool->{CameraModel} =~ /\bD50$/) {
                     $serial = 0x22; # D50 (ref 8)
+                } else {
+                    $serial = 0x60; # D200 (ref 10), D40X (ref PH)
                 }
             }
             $data = Decrypt(\$data, $serial, $count, $start, $len);
             my %subdirInfo = (
                 DataPt   => \$data,
                 DirStart => $offset,
-                DirLen   => length($data),
+                DirLen   => length($data) - $offset,
             );
             if ($verbose > 2) {
                 $exifTool->VerboseDir("Decrypted $$tagInfo{Name}");
@@ -1362,7 +1594,7 @@ Nikon maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -1379,6 +1611,8 @@ under the same terms as Perl itself.
 
 =item L<http://www.rottmerhusen.com/objektives/lensid/nikkor.html>
 
+=item L<http://www.wohlberg.net/public/software/photo/nstiffexif/>
+
 =back
 
 =head1 ACKNOWLEDGEMENTS
